package com.ruoyi.file.service;

import cn.hutool.core.io.FileUtil;
import com.ruoyi.common.exception.ASSERT;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.PathUtils;
import com.ruoyi.file.model.FileSystemFilter;
import com.ruoyi.file.model.FileModel;
import com.ruoyi.file.request.PutFileReq;
import org.apache.commons.io.FilenameUtils;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

@Service
public class FileSystemService
{
    public List<FileModel> ls(FileSystemFilter filter)
    {
        File[] files;
        if(StringUtils.isEmpty(filter.getFilePath()))
        {
            files = File.listRoots();
        }
        else
        {
            File file = new File(filter.getFilePath());
            ASSERT.EXP(file.exists(), "路径不存在: " + filter.getFilePath());
            ASSERT.EXP(file.isDirectory(), "路径非目录: " + filter.getFilePath());
            files = file.listFiles(filter);
        }

        List<FileModel> list = new ArrayList<>();
        if(null != files)
        {
            for(File f : files)
            {
                list.add(new FileModel(f));
            }
        }
        list.sort(new FileModel());
        return list;
    }

    public FileModel put(PutFileReq req, MultipartFile file)
    {
        String dst = PathUtils.appendPaths(req.getPath(), file.getOriginalFilename());
        File dstFile = new File(dst);
        if(dstFile.exists())
        {
            if(dstFile.isDirectory())
                throw new RuntimeException("目标文件是目录: " + dst);
            if(req.allowRename())
            {
                int i = 1;
                do
                {
                    String originalFilename = file.getOriginalFilename();
                    String extension = FilenameUtils.getExtension(originalFilename);
                    originalFilename = PathUtils.removeExtension(originalFilename);
                    originalFilename += "(" + i++ + ")";
                    if(StringUtils.isNotEmpty(extension))
                        originalFilename += "." + extension;
                    dst = PathUtils.appendPaths(req.getPath(), originalFilename);
                    dstFile = new File(dst);
                } while(dstFile.exists());
            }
            else if(req.allowOverride())
            {
                // override
                throw new RuntimeException("目标文件已存在, 不允许覆盖: " + dst);
            }
            else
                throw new RuntimeException("目标文件已存在: " + dst);
        }

        File parentFile = dstFile.getParentFile();
        if(null == parentFile)
            throw new RuntimeException("必须指定存储目录: " + dst);
        if(parentFile.exists() && !parentFile.isDirectory())
            throw new RuntimeException("存储目录必须是文件夹: " + dst);
        if(!parentFile.exists())
        {
            if(!parentFile.mkdirs())
                throw new RuntimeException("存储目录不存在且创建失败: " + dst);
        }
        try
        {
            byte[] bytes = file.getBytes();
            FileUtil.writeBytes(bytes, dstFile);
            return new FileModel(dstFile);
        }
        catch(Exception e)
        {
            e.printStackTrace();
            throw new RuntimeException(e);
        }
    }
}
